<?php

namespace Oro\Bundle\RedirectBundle\Generator;

use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Oro\Bundle\FrontendLocalizationBundle\Manager\UserLocalizationManager;
use Oro\Bundle\LocaleBundle\Entity\Localization;
use Oro\Bundle\OrganizationBundle\Entity\OrganizationAwareInterface;
use Oro\Bundle\RedirectBundle\Cache\UrlCacheInterface;
use Oro\Bundle\RedirectBundle\Entity\LocalizedSlugPrototypeAwareInterface;
use Oro\Bundle\RedirectBundle\Entity\Slug;
use Oro\Bundle\RedirectBundle\Entity\SluggableInterface;
use Oro\Bundle\RedirectBundle\Generator\DTO\SlugUrl;
use Oro\Bundle\RedirectBundle\Provider\RoutingInformationProviderInterface;
use Oro\Component\Routing\RouteData;

/**
 * Removes old caches
 * Generates new caches (only "slug" that were explicitly updated)
 */
class SlugEntityGenerator
{
    /**
     * @var RoutingInformationProviderInterface
     */
    protected $routingInformationProvider;

    /**
     * @var UniqueSlugResolver
     */
    protected $slugResolver;

    /**
     * @var UrlCacheInterface
     */
    private $urlCache;

    /**
     * @var RedirectGenerator
     */
    private $redirectGenerator;

    /**
     * @var UserLocalizationManager
     */
    private $userLocalizationManager;

    /**
     * @param RoutingInformationProviderInterface $routingInformationProvider
     * @param UniqueSlugResolver $slugResolver
     * @param RedirectGenerator $redirectGenerator
     * @param UrlCacheInterface $urlCache
     * @param UserLocalizationManager $userLocalizationManager
     */
    public function __construct(
        RoutingInformationProviderInterface $routingInformationProvider,
        UniqueSlugResolver $slugResolver,
        RedirectGenerator $redirectGenerator,
        UrlCacheInterface $urlCache,
        UserLocalizationManager $userLocalizationManager
    ) {
        $this->routingInformationProvider = $routingInformationProvider;
        $this->slugResolver = $slugResolver;
        $this->redirectGenerator = $redirectGenerator;
        $this->urlCache = $urlCache;
        $this->userLocalizationManager = $userLocalizationManager;
    }

    /**
     * @param SluggableInterface $entity
     * @param bool $generateRedirects
     */
    public function generate(SluggableInterface $entity, $generateRedirects = false)
    {
        $localizations = $this->userLocalizationManager->getEnabledLocalizations();
        $routeData = $this->routingInformationProvider->getRouteData($entity);
        foreach ($localizations as $localization) {
            $this->urlCache->removeUrl(
                $routeData->getRoute(),
                $routeData->getRouteParameters(),
                $this->getLocalizationId($localization)
            );
        }

        $slugUrls = $this->getResolvedSlugUrls($entity);

        /** @var Slug[] $toRemove */
        $toRemove = [];
        foreach ($entity->getSlugs() as $slug) {
            $localizationId = $this->getLocalizationId($slug->getLocalization());

            // Update existing
            if ($slugUrls->containsKey($localizationId)) {
                $slugUrl = $slugUrls->get($localizationId);

                $previousSlug = clone $slug;
                $updatedUrl = $slugUrl->getUrl();
                $slug->setUrl($updatedUrl);
                $slug->setSlugPrototype($slugUrl->getSlug());

                $this->redirectGenerator->updateRedirects($previousSlug->getUrl(), $slug);

                if ($generateRedirects) {
                    $this->redirectGenerator->generateForSlug($previousSlug, $slug);
                }

                $this->urlCache->setUrl(
                    $slug->getRouteName(),
                    $slug->getRouteParameters(),
                    $slug->getUrl(),
                    $slug->getSlugPrototype(),
                    $this->getLocalizationId($slug->getLocalization())
                );
            } else {
                $toRemove[] = $slug;
            }
        }

        // Remove deleted
        foreach ($toRemove as $slugToRemove) {
            $entity->removeSlug($slugToRemove);
        }

        // Add new
        foreach ($slugUrls as $slugUrl) {
            if ($this->getExistingSlugs($slugUrl, $entity->getSlugs())->isEmpty()) {
                $routeData = $this->routingInformationProvider->getRouteData($entity);
                $slug = $this->createSlug($routeData, $slugUrl);
                if ($entity instanceof OrganizationAwareInterface) {
                    $slug->setOrganization($entity->getOrganization());
                }

                $entity->addSlug($slug);

                $this->urlCache->setUrl(
                    $slug->getRouteName(),
                    $slug->getRouteParameters(),
                    $slug->getUrl(),
                    $slug->getSlugPrototype(),
                    $this->getLocalizationId($slug->getLocalization())
                );
            }
        }

        $this->updateSlugPrototypes($entity, $slugUrls);
    }

    /**
     * @param LocalizedSlugPrototypeAwareInterface $entity
     * @return array|SlugUrl[]
     */
    protected function getFilledSlugPrototypes(LocalizedSlugPrototypeAwareInterface $entity)
    {
        $slugPrototypes = $entity->getSlugPrototypes();
        $filledSlugPrototypes = [];
        foreach ($slugPrototypes as $slugPrototype) {
            $value = trim($slugPrototype->getString());
            // empty() function can not be used here, as '0' is a valid slug prototype value
            if ($value !== '' && $value !== null && !$slugPrototype->getFallback()) {
                $filledSlugPrototypes[] = new SlugUrl($value, $slugPrototype->getLocalization());
            }
        }

        return $filledSlugPrototypes;
    }

    /**
     * @param SluggableInterface $entity
     * @return Collection|SlugUrl[]
     */
    public function prepareSlugUrls(SluggableInterface $entity)
    {
        $filledSlugPrototypes = $this->getFilledSlugPrototypes($entity);
        $slugUrls = new ArrayCollection();
        foreach ($filledSlugPrototypes as $filledSlugPrototype) {
            $slugPrototype = $filledSlugPrototype->getUrl();
            $localization = $filledSlugPrototype->getLocalization();

            $url = $this->getUrl($entity, $slugPrototype);
            $slugUrl = new SlugUrl($url, $localization, $slugPrototype);
            $slugUrls->set($this->getLocalizationId($localization), $slugUrl);
        }

        return $slugUrls;
    }

    /**
     * @param SlugUrl $slugUrl
     * @param Collection $slugs
     * @return Collection|null|Slug[]
     */
    protected function getExistingSlugs(SlugUrl $slugUrl, Collection $slugs)
    {
        return $slugs->filter(
            function (Slug $slug) use ($slugUrl) {
                return $slugUrl->getUrl() === $slug->getUrl()
                    && $slugUrl->getLocalization() === $slug->getLocalization();
            }
        );
    }

    /**
     * @param SluggableInterface $entity
     * @param string $slugPrototype
     * @return string
     */
    protected function getUrl(SluggableInterface $entity, $slugPrototype)
    {
        $prefix = trim($this->routingInformationProvider->getUrlPrefix($entity), Slug::DELIMITER);

        $url = Slug::DELIMITER . $slugPrototype;
        if ($prefix) {
            $url = Slug::DELIMITER . $prefix . $url;
        }

        return $url;
    }

    /**
     * @param RouteData $routeData
     * @param SlugUrl $slugUrl
     * @return Slug
     */
    protected function createSlug(RouteData $routeData, SlugUrl $slugUrl)
    {
        $slug = new Slug();
        $slug->setUrl($slugUrl->getUrl());
        $slug->setSlugPrototype($slugUrl->getSlug());
        $slug->setLocalization($slugUrl->getLocalization());
        $slug->setRouteName($routeData->getRoute());
        $slug->setRouteParameters($routeData->getRouteParameters());

        return $slug;
    }

    /**
     * @param SluggableInterface $entity
     * @return Collection|SlugUrl[]
     */
    private function getResolvedSlugUrls(SluggableInterface $entity)
    {
        $slugUrls = $this->prepareSlugUrls($entity);
        $prefix = sprintf(
            '~^\%s?(%s\%s)?(.+)~',
            Slug::DELIMITER,
            preg_quote(trim($this->routingInformationProvider->getUrlPrefix($entity), Slug::DELIMITER), '~'),
            Slug::DELIMITER
        );

        foreach ($slugUrls as $slugUrl) {
            $url = $this->slugResolver->resolve($slugUrl, $entity);

            $slugUrl->setUrl($url);
            $slugUrl->setSlug(preg_replace($prefix, '$2', $url));
        }

        return $slugUrls;
    }

    /**
     * @param SluggableInterface $entity
     * @param SlugUrl[] $slugUrls
     */
    private function updateSlugPrototypes(SluggableInterface $entity, $slugUrls)
    {
        $slugPrototypesByLocalizationId = [];
        foreach ($entity->getSlugPrototypes() as $slugPrototype) {
            $slugPrototypesByLocalizationId[$this->getLocalizationId($slugPrototype->getLocalization())] =
                $slugPrototype;
        }

        foreach ($slugUrls as $localizationId => $slugUrl) {
            if (isset($slugPrototypesByLocalizationId[$localizationId])) {
                $slugPrototypesByLocalizationId[$localizationId]->setString($slugUrl->getSlug());
            }
        }
    }

    /**
     * @param Localization|null $localization
     * @return int
     */
    private function getLocalizationId(Localization $localization = null)
    {
        if ($localization) {
            return $localization->getId();
        }

        return 0;
    }
}
